'use strict';

var path = require('path');

/**
 * The preset is based on create-react-app, but with support for 'server'.
 * https://github.com/facebookincubator/create-react-app/tree/master/packages/babel-preset-react-app
 */
const plugins = [
  // Optimize lodash modules being used
  require.resolve('babel-plugin-lodash'),
  // class { handleClick = () => { } }
  require.resolve('babel-plugin-transform-class-properties'),
  // { ...state, completed: true }
  require.resolve('babel-plugin-transform-object-rest-spread', {
    useBuiltIns: true
  }),
  // Transforms JSX - Added so object-rest-spread in JSX uses builtIn
  [require.resolve('babel-plugin-transform-react-jsx'), {
    useBuiltIns: true
  }],
  // Allows Babel to resolve modules in ./src;
  [require.resolve('babel-plugin-module-resolver'),  {
    "root": ["./src"],
  }],
  // Polyfills the runtime needed for async/await and generators
  [require.resolve('babel-plugin-transform-runtime'), {
    helpers: false,
    polyfill: false,
    regenerator: true,
    // Resolve the Babel runtime relative to the config.
    moduleName: path.dirname(require.resolve('babel-runtime/package'))
  }],
];

// This is similar to how `env` works in Babel:
// https://babeljs.io/docs/usage/babelrc/#env-option
// We are not using `env` because it’s ignored in versions > babel-core@6.10.4:
// https://github.com/babel/babel/issues/4539
// https://github.com/facebookincubator/create-react-app/issues/720
// It’s also nice that we can enforce `NODE_ENV` being specified.
var env = process.env.BABEL_ENV || process.env.NODE_ENV;
if (env !== 'development' && env !== 'test' && env !== 'production' && env != 'server') {
  throw new Error(
    'Using `babel-preset-react-app` requires that you specify `NODE_ENV` or '+
    '`BABEL_ENV` environment variables. Valid values are "development", ' +
    '"test", and "production". Instead, received: ' + JSON.stringify(env) + '.'
  );
}

if (env === 'development' || env === 'test') {
  // The following two plugins are currently necessary to make jsx developer
  // friendly messages, if you wondering why it is added here separately rather
  // it comes with babel-preset-react see the below threads for more info
  // https://github.com/babel/babel/issues/4702
  // https://github.com/babel/babel/pull/3540#issuecomment-228673661
  // https://github.com/facebookincubator/create-react-app/issues/989
  plugins.push.apply(plugins, [
    // Adds component stack to warning messages
    require.resolve('babel-plugin-transform-react-jsx-source'),
    // Adds __self attribute to JSX which React will use for some warnings
    require.resolve('babel-plugin-transform-react-jsx-self')
  ]);
}

// Transform import statements to supported version
if (env === 'server' || env === 'test') {
  plugins.push.apply(plugins, [
    require.resolve('babel-plugin-dynamic-import-node'),
  ]);
} else {
  plugins.push.apply(plugins, [
    require.resolve('babel-plugin-dynamic-import-webpack'),
  ]);
}

module.exports = function(context, opts) {
  var config;
  require.resolve('babel-preset-latest');

  if (env === 'test') {
    config = {
      presets: [
        // ES features necessary for user's Node version
        [require('babel-preset-env').default, {
          targets: {
            node: 'current'
          },
          include: ['transform-es2015-destructuring']
        }],
        // JSX, Flow
        require.resolve('babel-preset-react')
      ],
      plugins: plugins
    };
  } else {
    config = {
      presets: [
        // Latest stable ECMAScript features
        ['latest', opts],
        // JSX, Flow
        require.resolve('babel-preset-react')
      ],
      plugins: plugins.concat([
        // function* () { yield 42; yield 43; }
        [require.resolve('babel-plugin-transform-regenerator'), {
          // Async functions are converted to generators by babel-preset-latest
          async: false
        }],
      ])
    };

    if (env === 'production') {
      plugins.push.apply(plugins, [
        // Remove PropTypes in Production
        require.resolve('babel-plugin-transform-react-remove-prop-types'),
        // Convert pure classes to functions
        require.resolve('babel-plugin-transform-react-pure-class-to-function'),
      ]);
    }
  }

  return config;
};